import json
from typing import Any, Dict, List, Tuple, Type, TypeVar, Union

import attr

from ..models.claim_metadata import ClaimMetadata
from ..types import UNSET, Unset

T = TypeVar("T", bound="ClaimCreateRequest")


@attr.s(auto_attribs=True)
class ClaimCreateRequest:
    """
    Attributes:
        product (str):
        network (str):
        url (str):
        user_requester (Union[Unset, None, str]):
        item_metadata (Union[Unset, ClaimMetadata]):
    """

    product: str
    network: str
    url: str
    user_requester: Union[Unset, None, str] = UNSET
    item_metadata: Union[Unset, ClaimMetadata] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        product = self.product
        network = self.network
        url = self.url
        user_requester = self.user_requester
        item_metadata: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.item_metadata, Unset):
            item_metadata = self.item_metadata.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "product": product,
                "network": network,
                "url": url,
            }
        )
        if user_requester is not UNSET:
            field_dict["user_requester"] = user_requester
        if item_metadata is not UNSET:
            field_dict["item_metadata"] = item_metadata

        return field_dict

    def to_multipart(self) -> Dict[str, Any]:
        product = self.product if isinstance(self.product, Unset) else (None, str(self.product).encode(), "text/plain")
        network = self.network if isinstance(self.network, Unset) else (None, str(self.network).encode(), "text/plain")
        url = self.url if isinstance(self.url, Unset) else (None, str(self.url).encode(), "text/plain")
        user_requester = (
            self.user_requester
            if isinstance(self.user_requester, Unset)
            else (None, str(self.user_requester).encode(), "text/plain")
        )
        item_metadata: Union[Unset, Tuple[None, bytes, str]] = UNSET
        if not isinstance(self.item_metadata, Unset):
            item_metadata = (None, json.dumps(self.item_metadata.to_dict()).encode(), "application/json")

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {key: (None, str(value).encode(), "text/plain") for key, value in self.additional_properties.items()}
        )
        field_dict.update(
            {
                "product": product,
                "network": network,
                "url": url,
            }
        )
        if user_requester is not UNSET:
            field_dict["user_requester"] = user_requester
        if item_metadata is not UNSET:
            field_dict["item_metadata"] = item_metadata

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        product = d.pop("product")

        network = d.pop("network")

        url = d.pop("url")

        user_requester = d.pop("user_requester", UNSET)

        _item_metadata = d.pop("item_metadata", UNSET)
        item_metadata: Union[Unset, ClaimMetadata]
        if isinstance(_item_metadata, Unset):
            item_metadata = UNSET
        else:
            item_metadata = ClaimMetadata.from_dict(_item_metadata)

        claim_create_request = cls(
            product=product,
            network=network,
            url=url,
            user_requester=user_requester,
            item_metadata=item_metadata,
        )

        claim_create_request.additional_properties = d
        return claim_create_request

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



""" Contains all the data models used in inputs/outputs """

from .activity import Activity
from .blank_enum import BlankEnum
from .case import Case
from .claim_create_request import ClaimCreateRequest
from .claim_metadata import ClaimMetadata
from .claim_response import ClaimResponse
from .claim_retrieve import ClaimRetrieve
from .country_enum import CountryEnum
from .create_product_request import CreateProductRequest
from .create_product_response import CreateProductResponse
from .create_product_service_request import CreateProductServiceRequest
from .current_state_enum import CurrentStateEnum
from .customer_create_request import CustomerCreateRequest
from .customer_create_response import CustomerCreateResponse
from .product_metadata import ProductMetadata
from .schema_retrieve_format import SchemaRetrieveFormat
from .schema_retrieve_lang import SchemaRetrieveLang
from .schema_retrieve_response_200 import SchemaRetrieveResponse200
from .service import Service



import datetime
from typing import Any, Dict, List, Type, TypeVar

import attr
from dateutil.parser import isoparse

from ..models.case import Case

T = TypeVar("T", bound="Service")


@attr.s(auto_attribs=True)
class Service:
    """
    Attributes:
        id (str):
        code (str):
        cases (List[Case]):
        created_at (datetime.datetime):
    """

    id: str
    code: str
    cases: List[Case]
    created_at: datetime.datetime
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id
        code = self.code
        cases = []
        for cases_item_data in self.cases:
            cases_item = cases_item_data.to_dict()

            cases.append(cases_item)

        created_at = self.created_at.isoformat()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "code": code,
                "cases": cases,
                "created_at": created_at,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id")

        code = d.pop("code")

        cases = []
        _cases = d.pop("cases")
        for cases_item_data in _cases:
            cases_item = Case.from_dict(cases_item_data)

            cases.append(cases_item)

        created_at = isoparse(d.pop("created_at"))

        service = cls(
            id=id,
            code=code,
            cases=cases,
            created_at=created_at,
        )

        service.additional_properties = d
        return service

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import Any, Dict, List, Optional, Union, cast

import httpx

from legaltech_api_client.client import AuthenticatedClient
from legaltech_api_client.models.activity import Activity
from legaltech_api_client.types import UNSET, Response, Unset


def _get_kwargs(
    claim_id: str,
    *,
    client: AuthenticatedClient,
    created_at: Union[Unset, None, datetime.datetime] = UNSET,
    created_at_gt: Union[Unset, None, datetime.datetime] = UNSET,
    created_at_gte: Union[Unset, None, datetime.datetime] = UNSET,
    created_at_lt: Union[Unset, None, datetime.datetime] = UNSET,
    created_at_lte: Union[Unset, None, datetime.datetime] = UNSET,
) -> Dict[str, Any]:
    url = "{}/api/v1/claims/{claim_id}/activities/".format(client.base_url, claim_id=claim_id)

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    params: Dict[str, Any] = {}
    json_created_at: Union[Unset, None, str] = UNSET
    if not isinstance(created_at, Unset):
        json_created_at = created_at.isoformat() if created_at else None

    params["created_at"] = json_created_at

    json_created_at_gt: Union[Unset, None, str] = UNSET
    if not isinstance(created_at_gt, Unset):
        json_created_at_gt = created_at_gt.isoformat() if created_at_gt else None

    params["created_at__gt"] = json_created_at_gt

    json_created_at_gte: Union[Unset, None, str] = UNSET
    if not isinstance(created_at_gte, Unset):
        json_created_at_gte = created_at_gte.isoformat() if created_at_gte else None

    params["created_at__gte"] = json_created_at_gte

    json_created_at_lt: Union[Unset, None, str] = UNSET
    if not isinstance(created_at_lt, Unset):
        json_created_at_lt = created_at_lt.isoformat() if created_at_lt else None

    params["created_at__lt"] = json_created_at_lt

    json_created_at_lte: Union[Unset, None, str] = UNSET
    if not isinstance(created_at_lte, Unset):
        json_created_at_lte = created_at_lte.isoformat() if created_at_lte else None

    params["created_at__lte"] = json_created_at_lte

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "method": "get",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "params": params,
    }


def _parse_response(*, response: httpx.Response) -> Optional[Union[Any, List[Activity]]]:
    if response.status_code == 201:
        response_201 = []
        _response_201 = response.json()
        for response_201_item_data in _response_201:
            response_201_item = Activity.from_dict(response_201_item_data)

            response_201.append(response_201_item)

        return response_201
    if response.status_code == 400:
        response_400 = cast(Any, None)
        return response_400
    if response.status_code == 404:
        response_404 = cast(Any, None)
        return response_404
    if response.status_code == 403:
        response_403 = cast(Any, None)
        return response_403
    return None


def _build_response(*, response: httpx.Response) -> Response[Union[Any, List[Activity]]]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    claim_id: str,
    *,
    client: AuthenticatedClient,
    created_at: Union[Unset, None, datetime.datetime] = UNSET,
    created_at_gt: Union[Unset, None, datetime.datetime] = UNSET,
    created_at_gte: Union[Unset, None, datetime.datetime] = UNSET,
    created_at_lt: Union[Unset, None, datetime.datetime] = UNSET,
    created_at_lte: Union[Unset, None, datetime.datetime] = UNSET,
) -> Response[Union[Any, List[Activity]]]:
    """Get activities perform on a Claim.

    Args:
        claim_id (str):
        client (AuthenticatedClient):
        created_at (Union[Unset, None, datetime.datetime]):
        created_at_gt (Union[Unset, None, datetime.datetime]):
        created_at_gte (Union[Unset, None, datetime.datetime]):
        created_at_lt (Union[Unset, None, datetime.datetime]):
        created_at_lte (Union[Unset, None, datetime.datetime]):

    Returns:
        Response[Union[Any, List[Activity]]]
    """

    kwargs = _get_kwargs(
        claim_id=claim_id,
        client=client,
        created_at=created_at,
        created_at_gt=created_at_gt,
        created_at_gte=created_at_gte,
        created_at_lt=created_at_lt,
        created_at_lte=created_at_lte,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    claim_id: str,
    *,
    client: AuthenticatedClient,
    created_at: Union[Unset, None, datetime.datetime] = UNSET,
    created_at_gt: Union[Unset, None, datetime.datetime] = UNSET,
    created_at_gte: Union[Unset, None, datetime.datetime] = UNSET,
    created_at_lt: Union[Unset, None, datetime.datetime] = UNSET,
    created_at_lte: Union[Unset, None, datetime.datetime] = UNSET,
) -> Optional[Union[Any, List[Activity]]]:
    """Get activities perform on a Claim.

    Args:
        claim_id (str):
        client (AuthenticatedClient):
        created_at (Union[Unset, None, datetime.datetime]):
        created_at_gt (Union[Unset, None, datetime.datetime]):
        created_at_gte (Union[Unset, None, datetime.datetime]):
        created_at_lt (Union[Unset, None, datetime.datetime]):
        created_at_lte (Union[Unset, None, datetime.datetime]):

    Returns:
        Response[Union[Any, List[Activity]]]
    """

    return sync_detailed(
        claim_id=claim_id,
        client=client,
        created_at=created_at,
        created_at_gt=created_at_gt,
        created_at_gte=created_at_gte,
        created_at_lt=created_at_lt,
        created_at_lte=created_at_lte,
    ).parsed


async def asyncio_detailed(
    claim_id: str,
    *,
    client: AuthenticatedClient,
    created_at: Union[Unset, None, datetime.datetime] = UNSET,
    created_at_gt: Union[Unset, None, datetime.datetime] = UNSET,
    created_at_gte: Union[Unset, None, datetime.datetime] = UNSET,
    created_at_lt: Union[Unset, None, datetime.datetime] = UNSET,
    created_at_lte: Union[Unset, None, datetime.datetime] = UNSET,
) -> Response[Union[Any, List[Activity]]]:
    """Get activities perform on a Claim.

    Args:
        claim_id (str):
        client (AuthenticatedClient):
        created_at (Union[Unset, None, datetime.datetime]):
        created_at_gt (Union[Unset, None, datetime.datetime]):
        created_at_gte (Union[Unset, None, datetime.datetime]):
        created_at_lt (Union[Unset, None, datetime.datetime]):
        created_at_lte (Union[Unset, None, datetime.datetime]):

    Returns:
        Response[Union[Any, List[Activity]]]
    """

    kwargs = _get_kwargs(
        claim_id=claim_id,
        client=client,
        created_at=created_at,
        created_at_gt=created_at_gt,
        created_at_gte=created_at_gte,
        created_at_lt=created_at_lt,
        created_at_lte=created_at_lte,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(response=response)


async def asyncio(
    claim_id: str,
    *,
    client: AuthenticatedClient,
    created_at: Union[Unset, None, datetime.datetime] = UNSET,
    created_at_gt: Union[Unset, None, datetime.datetime] = UNSET,
    created_at_gte: Union[Unset, None, datetime.datetime] = UNSET,
    created_at_lt: Union[Unset, None, datetime.datetime] = UNSET,
    created_at_lte: Union[Unset, None, datetime.datetime] = UNSET,
) -> Optional[Union[Any, List[Activity]]]:
    """Get activities perform on a Claim.

    Args:
        claim_id (str):
        client (Client):
        created_at (Union[Unset, None, datetime.datetime]):
        created_at_gt (Union[Unset, None, datetime.datetime]):
        created_at_gte (Union[Unset, None, datetime.datetime]):
        created_at_lt (Union[Unset, None, datetime.datetime]):
        created_at_lte (Union[Unset, None, datetime.datetime]):

    Returns:
        Response[Union[Any, List[Activity]]]
    """

    return (
        await asyncio_detailed(
            claim_id=claim_id,
            client=client,
            created_at=created_at,
            created_at_gt=created_at_gt,
            created_at_gte=created_at_gte,
            created_at_lt=created_at_lt,
            created_at_lte=created_at_lte,
        )
    ).parsed



""" Contains some shared types for properties """
from typing import Generic, MutableMapping, Optional, TypeVar

import attr


class Unset:
    def __bool__(self) -> bool:
        return False


UNSET: Unset = Unset()


T = TypeVar("T")


@attr.s(auto_attribs=True)
class Response(Generic[T]):
    """A response from an endpoint"""

    status_code: int
    content: bytes
    headers: MutableMapping[str, str]
    parsed: Optional[T]


__all__ = ["Response", ]



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="ClaimMetadata")


@attr.s(auto_attribs=True)
class ClaimMetadata:
    """
    Attributes:
        item_country (Union[Unset, str]):
        item_title (Union[Unset, str]):
        item_vendor (Union[Unset, str]):
        item_price (Union[Unset, str]):
        item_currency (Union[Unset, str]):
        item_description (Union[Unset, str]):
        item_ref (Union[Unset, str]):
        user_verified (Union[Unset, bool]):
        screenshot (Union[Unset, str]):
        date_captured (Union[Unset, str]):
    """

    item_country: Union[Unset, str] = UNSET
    item_title: Union[Unset, str] = UNSET
    item_vendor: Union[Unset, str] = UNSET
    item_price: Union[Unset, str] = UNSET
    item_currency: Union[Unset, str] = UNSET
    item_description: Union[Unset, str] = UNSET
    item_ref: Union[Unset, str] = UNSET
    user_verified: Union[Unset, bool] = UNSET
    screenshot: Union[Unset, str] = UNSET
    date_captured: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        item_country = self.item_country
        item_title = self.item_title
        item_vendor = self.item_vendor
        item_price = self.item_price
        item_currency = self.item_currency
        item_description = self.item_description
        item_ref = self.item_ref
        user_verified = self.user_verified
        screenshot = self.screenshot
        date_captured = self.date_captured

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if item_country is not UNSET:
            field_dict["item_country"] = item_country
        if item_title is not UNSET:
            field_dict["item_title"] = item_title
        if item_vendor is not UNSET:
            field_dict["item_vendor"] = item_vendor
        if item_price is not UNSET:
            field_dict["item_price"] = item_price
        if item_currency is not UNSET:
            field_dict["item_currency"] = item_currency
        if item_description is not UNSET:
            field_dict["item_description"] = item_description
        if item_ref is not UNSET:
            field_dict["item_ref"] = item_ref
        if user_verified is not UNSET:
            field_dict["user_verified"] = user_verified
        if screenshot is not UNSET:
            field_dict["screenshot"] = screenshot
        if date_captured is not UNSET:
            field_dict["date_captured"] = date_captured

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        item_country = d.pop("item_country", UNSET)

        item_title = d.pop("item_title", UNSET)

        item_vendor = d.pop("item_vendor", UNSET)

        item_price = d.pop("item_price", UNSET)

        item_currency = d.pop("item_currency", UNSET)

        item_description = d.pop("item_description", UNSET)

        item_ref = d.pop("item_ref", UNSET)

        user_verified = d.pop("user_verified", UNSET)

        screenshot = d.pop("screenshot", UNSET)

        date_captured = d.pop("date_captured", UNSET)

        claim_metadata = cls(
            item_country=item_country,
            item_title=item_title,
            item_vendor=item_vendor,
            item_price=item_price,
            item_currency=item_currency,
            item_description=item_description,
            item_ref=item_ref,
            user_verified=user_verified,
            screenshot=screenshot,
            date_captured=date_captured,
        )

        claim_metadata.additional_properties = d
        return claim_metadata

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Optional, Union, cast

import httpx

from legaltech_api_client.client import AuthenticatedClient
from legaltech_api_client.models.customer_create_request import CustomerCreateRequest
from legaltech_api_client.models.customer_create_response import CustomerCreateResponse
from legaltech_api_client.types import Response


def _get_kwargs(
        *,
        client: AuthenticatedClient,
        data: CustomerCreateRequest,
) -> Dict[str, Any]:
    url = "{}/api/v1/customers/".format(client.base_url)

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    return {
        "method": "post",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "json": data.to_dict(),
    }


def _parse_response(*, response: httpx.Response) -> Optional[Union[Any, CustomerCreateResponse]]:
    if response.status_code == 200:
        response_200 = CustomerCreateResponse.from_dict(response.json())

        return response_200
    if response.status_code == 400:
        response_400 = cast(Any, None)
        return response_400
    if response.status_code == 409:
        response_409 = cast(Any, None)
        return response_409
    if response.status_code == 403:
        response_403 = cast(Any, None)
        return response_403
    return None


def _build_response(*, response: httpx.Response) -> Response[Union[Any, CustomerCreateResponse]]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
        *,
        client: AuthenticatedClient,
        data: CustomerCreateRequest,
) -> Response[Union[Any, CustomerCreateResponse]]:
    """Create new Customer.

    Args:
        client (Client):
        data (CustomerCreateRequest):

    Returns:
        Response[Union[Any, CustomerCreateResponse]]
    """

    kwargs = _get_kwargs(
        client=client,
        data=data,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


def sync(
        *,
        client: AuthenticatedClient,
        data: CustomerCreateRequest,
) -> Optional[Union[Any, CustomerCreateResponse]]:
    """Create new Customer.

    Args:
        client (AuthenticatedClient):
        data (CustomerCreateRequest):

    Returns:
        Response[Union[Any, CustomerCreateResponse]]
    """

    return sync_detailed(
        client=client,
        data=data,
    ).parsed


async def asyncio_detailed(
        *,
        client: AuthenticatedClient,
        data: CustomerCreateRequest,
) -> Response[Union[Any, CustomerCreateResponse]]:
    """Create new Customer.

    Args:
        client (AuthenticatedClient):
        data (CustomerCreateRequest):

    Returns:
        Response[Union[Any, CustomerCreateResponse]]
    """

    kwargs = _get_kwargs(
        client=client,
        data=data,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(response=response)


async def asyncio(
        *,
        client: AuthenticatedClient,
        data: CustomerCreateRequest,
) -> Optional[Union[Any, CustomerCreateResponse]]:
    """Create new Customer.

    Args:
        client (AuthenticatedClient):
        data (CustomerCreateRequest):

    Returns:
        Response[Union[Any, CustomerCreateResponse]]
    """

    return (
        await asyncio_detailed(
            client=client,
            data=data,
        )
    ).parsed



import json
from typing import Any, Dict, List, Tuple, Type, TypeVar, Union

import attr

from ..models.create_product_service_request import CreateProductServiceRequest
from ..models.product_metadata import ProductMetadata
from ..types import UNSET, Unset

T = TypeVar("T", bound="CreateProductRequest")


@attr.s(auto_attribs=True)
class CreateProductRequest:
    """
    Attributes:
        customer (str):
        name (str):
        alias (str):
        service_data (CreateProductServiceRequest):
        metadata (Union[Unset, ProductMetadata]):
    """

    customer: str
    name: str
    alias: str
    service_data: CreateProductServiceRequest
    metadata: Union[Unset, ProductMetadata] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        customer = self.customer
        name = self.name
        alias = self.alias
        service_data = self.service_data.to_dict()

        metadata: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.metadata, Unset):
            metadata = self.metadata.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "customer": customer,
                "name": name,
                "alias": alias,
                "service_data": service_data,
            }
        )
        if metadata is not UNSET:
            field_dict["metadata"] = metadata

        return field_dict

    def to_multipart(self) -> Dict[str, Any]:
        customer = (
            self.customer if isinstance(self.customer, Unset) else (None, str(self.customer).encode(), "text/plain")
        )
        name = self.name if isinstance(self.name, Unset) else (None, str(self.name).encode(), "text/plain")
        alias = self.alias if isinstance(self.alias, Unset) else (None, str(self.alias).encode(), "text/plain")
        service_data = (None, json.dumps(self.service_data.to_dict()).encode(), "application/json")

        metadata: Union[Unset, Tuple[None, bytes, str]] = UNSET
        if not isinstance(self.metadata, Unset):
            metadata = (None, json.dumps(self.metadata.to_dict()).encode(), "application/json")

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {key: (None, str(value).encode(), "text/plain") for key, value in self.additional_properties.items()}
        )
        field_dict.update(
            {
                "customer": customer,
                "name": name,
                "alias": alias,
                "service_data": service_data,
            }
        )
        if metadata is not UNSET:
            field_dict["metadata"] = metadata

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        customer = d.pop("customer")

        name = d.pop("name")

        alias = d.pop("alias")

        service_data = CreateProductServiceRequest.from_dict(d.pop("service_data"))

        _metadata = d.pop("metadata", UNSET)
        metadata: Union[Unset, ProductMetadata]
        if isinstance(_metadata, Unset):
            metadata = UNSET
        else:
            metadata = ProductMetadata.from_dict(_metadata)

        create_product_request = cls(
            customer=customer,
            name=name,
            alias=alias,
            service_data=service_data,
            metadata=metadata,
        )

        create_product_request.additional_properties = d
        return create_product_request

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class CurrentStateEnum(str, Enum):
    CLAIM_CREATED = "claim_created"
    CLAIM_MISMATCH_DOC = "claim_mismatch_doc"
    CLAIM_NOT_SCOPE = "claim_not_scope"
    CLAIM_REJECTED = "claim_rejected"
    CLAIM_RESTORED = "claim_restored"
    CONTENT_BRAND_ABUSE = "content_brand_abuse"
    CONTENT_COUNTERFEIT = "content_counterfeit"
    CONTENT_DISCARDED = "content_discarded"
    CONTENT_DOWN = "content_down"
    CONTENT_GRAY_MARKET = "content_gray_market"
    CONTENT_LEGAL = "content_legal"
    CONTENT_NOT_ENFORCEABLE = "content_not_enforceable"
    CONTENT_PARALLEL_IMPORT = "content_parallel_import"
    CONTENT_TO_WHITELIST = "content_to_whitelist"
    CONTENT_USE_OF_IMAGES = "content_use_of_images"
    LETTER_SENT = "letter_sent"
    NOTIFICATION_SENT = "notification_sent"
    OPEN_DISPUTE = "open_dispute"
    PENDING_LEGAL_SETUP = "pending_legal_setup"
    TEST_PURCHASE = "test_purchase"
    TEST_PURCHASE_RESULTS = "test_purchase_results"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..models.blank_enum import BlankEnum
from ..models.country_enum import CountryEnum
from ..types import UNSET, Unset

T = TypeVar("T", bound="CustomerCreateRequest")


@attr.s(auto_attribs=True)
class CustomerCreateRequest:
    """
    Attributes:
        name (str):
        email (str):
        matrix_company (Union[Unset, None, str]):
        country (Union[BlankEnum, CountryEnum, None, Unset]):
    """

    name: str
    email: str
    matrix_company: Union[Unset, None, str] = UNSET
    country: Union[BlankEnum, CountryEnum, None, Unset] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        name = self.name
        email = self.email
        matrix_company = self.matrix_company
        country: Union[None, Unset, str]
        if isinstance(self.country, Unset):
            country = UNSET
        elif self.country is None:
            country = None

        elif isinstance(self.country, CountryEnum):
            country = UNSET
            if not isinstance(self.country, Unset):
                country = self.country.value

        elif isinstance(self.country, BlankEnum):
            country = UNSET
            if not isinstance(self.country, Unset):
                country = self.country.value

        else:
            country = self.country

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
                "email": email,
            }
        )
        if matrix_company is not UNSET:
            field_dict["matrix_company"] = matrix_company
        if country is not UNSET:
            field_dict["country"] = country

        return field_dict

    def to_multipart(self) -> Dict[str, Any]:
        name = self.name if isinstance(self.name, Unset) else (None, str(self.name).encode(), "text/plain")
        email = self.email if isinstance(self.email, Unset) else (None, str(self.email).encode(), "text/plain")
        matrix_company = (
            self.matrix_company
            if isinstance(self.matrix_company, Unset)
            else (None, str(self.matrix_company).encode(), "text/plain")
        )
        country: Union[None, Unset, str]
        if isinstance(self.country, Unset):
            country = UNSET
        elif self.country is None:
            country = None

        elif isinstance(self.country, CountryEnum):
            country = UNSET
            if not isinstance(self.country, Unset):
                country = (None, str(self.country.value).encode(), "text/plain")

        elif isinstance(self.country, BlankEnum):
            country = UNSET
            if not isinstance(self.country, Unset):
                country = (None, str(self.country.value).encode(), "text/plain")

        else:
            country = self.country

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {key: (None, str(value).encode(), "text/plain") for key, value in self.additional_properties.items()}
        )
        field_dict.update(
            {
                "name": name,
                "email": email,
            }
        )
        if matrix_company is not UNSET:
            field_dict["matrix_company"] = matrix_company
        if country is not UNSET:
            field_dict["country"] = country

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        name = d.pop("name")

        email = d.pop("email")

        matrix_company = d.pop("matrix_company", UNSET)

        def _parse_country(data: object) -> Union[BlankEnum, CountryEnum, None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                _country_type_0 = data
                country_type_0: Union[Unset, CountryEnum]
                if isinstance(_country_type_0, Unset):
                    country_type_0 = UNSET
                else:
                    country_type_0 = CountryEnum(_country_type_0)

                return country_type_0
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, str):
                    raise TypeError()
                _country_type_1 = data
                country_type_1: Union[Unset, BlankEnum]
                if isinstance(_country_type_1, Unset):
                    country_type_1 = UNSET
                else:
                    country_type_1 = BlankEnum(_country_type_1)

                return country_type_1
            except:  # noqa: E722
                pass
            return cast(Union[BlankEnum, CountryEnum, None, Unset], data)

        country = _parse_country(d.pop("country", UNSET))

        customer_create_request = cls(
            name=name,
            email=email,
            matrix_company=matrix_company,
            country=country,
        )

        customer_create_request.additional_properties = d
        return customer_create_request

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import Any, Dict, List, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..models.claim_metadata import ClaimMetadata
from ..models.current_state_enum import CurrentStateEnum
from ..types import UNSET, Unset

T = TypeVar("T", bound="ClaimRetrieve")


@attr.s(auto_attribs=True)
class ClaimRetrieve:
    """
    Attributes:
        id (str):
        product_id (str):
        network (str):
        url (str):
        is_assigned (str):
        current_state (Union[Unset, CurrentStateEnum]):
        user_requester (Union[Unset, None, str]):
        item_metadata (Union[Unset, ClaimMetadata]):
        notes (Union[Unset, None, str]):
        assignment_at (Union[Unset, None, datetime.datetime]):
    """

    id: str
    product_id: str
    network: str
    url: str
    is_assigned: str
    current_state: Union[Unset, CurrentStateEnum] = UNSET
    user_requester: Union[Unset, None, str] = UNSET
    item_metadata: Union[Unset, ClaimMetadata] = UNSET
    notes: Union[Unset, None, str] = UNSET
    assignment_at: Union[Unset, None, datetime.datetime] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id
        product_id = self.product_id
        network = self.network
        url = self.url
        is_assigned = self.is_assigned
        current_state: Union[Unset, str] = UNSET
        if not isinstance(self.current_state, Unset):
            current_state = self.current_state.value

        user_requester = self.user_requester
        item_metadata: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.item_metadata, Unset):
            item_metadata = self.item_metadata.to_dict()

        notes = self.notes
        assignment_at: Union[Unset, None, str] = UNSET
        if not isinstance(self.assignment_at, Unset):
            assignment_at = self.assignment_at.isoformat() if self.assignment_at else None

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "product_id": product_id,
                "network": network,
                "url": url,
                "is_assigned": is_assigned,
            }
        )
        if current_state is not UNSET:
            field_dict["current_state"] = current_state
        if user_requester is not UNSET:
            field_dict["user_requester"] = user_requester
        if item_metadata is not UNSET:
            field_dict["item_metadata"] = item_metadata
        if notes is not UNSET:
            field_dict["notes"] = notes
        if assignment_at is not UNSET:
            field_dict["assignment_at"] = assignment_at

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id")

        product_id = d.pop("product_id")

        network = d.pop("network")

        url = d.pop("url")

        is_assigned = d.pop("is_assigned")

        _current_state = d.pop("current_state", UNSET)
        current_state: Union[Unset, CurrentStateEnum]
        if isinstance(_current_state, Unset):
            current_state = UNSET
        else:
            current_state = CurrentStateEnum(_current_state)

        user_requester = d.pop("user_requester", UNSET)

        _item_metadata = d.pop("item_metadata", UNSET)
        item_metadata: Union[Unset, ClaimMetadata]
        if isinstance(_item_metadata, Unset):
            item_metadata = UNSET
        else:
            item_metadata = ClaimMetadata.from_dict(_item_metadata)

        notes = d.pop("notes", UNSET)

        _assignment_at = d.pop("assignment_at", UNSET)
        assignment_at: Union[Unset, None, datetime.datetime]
        if _assignment_at is None:
            assignment_at = None
        elif isinstance(_assignment_at, Unset):
            assignment_at = UNSET
        else:
            assignment_at = isoparse(_assignment_at)

        claim_retrieve = cls(
            id=id,
            product_id=product_id,
            network=network,
            url=url,
            is_assigned=is_assigned,
            current_state=current_state,
            user_requester=user_requester,
            item_metadata=item_metadata,
            notes=notes,
            assignment_at=assignment_at,
        )

        claim_retrieve.additional_properties = d
        return claim_retrieve

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import Any, Dict, List, Type, TypeVar

import attr
from dateutil.parser import isoparse

T = TypeVar("T", bound="CreateProductServiceRequest")


@attr.s(auto_attribs=True)
class CreateProductServiceRequest:
    """
    Attributes:
        service (str):
        date_start (datetime.date):
        date_end (datetime.date):
    """

    service: str
    date_start: datetime.date
    date_end: datetime.date
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        service = self.service
        date_start = self.date_start.isoformat()
        date_end = self.date_end.isoformat()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "service": service,
                "date_start": date_start,
                "date_end": date_end,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        service = d.pop("service")

        date_start = isoparse(d.pop("date_start")).date()

        date_end = isoparse(d.pop("date_end")).date()

        create_product_service_request = cls(
            service=service,
            date_start=date_start,
            date_end=date_end,
        )

        create_product_service_request.additional_properties = d
        return create_product_service_request

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
